@@ -457,30 +457,30 @@ nautilus_bonobo_remove_menu_items_and_commands (BonoboUIComponent *ui,
 	g_free (remove_wildcard);
 }
 
-/* Call to set the user-visible label of a menu item to a string
+/* Call to set the user-visible label of a command to a string
  * containing an underscore accelerator. The underscore is stripped
- * off before setting the label of the command, because pop-up menu
- * and toolbar button labels shouldn't have the underscore.
+ * off before setting the label of the toolitem, because toolbar
+ * button labels shouldn't have the underscore.
  */
 void	 
-nautilus_bonobo_set_label_for_menu_item_and_command (BonoboUIComponent *ui,
-						     const char	*menu_item_path,
-						     const char	*command_path,
-						     const char	*label_with_underscore)
+nautilus_bonobo_set_label_for_toolitem_and_command (BonoboUIComponent *ui,
+						    const char	*toolitem_path,
+						    const char	*command_path,
+						    const char	*label_with_underscore)
 {
 	char *label_no_underscore;
 
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (ui));
-	g_return_if_fail (menu_item_path != NULL);
+	g_return_if_fail (toolitem_path != NULL);
 	g_return_if_fail (command_path != NULL);
 	g_return_if_fail (label_with_underscore != NULL);
 
 	label_no_underscore = eel_str_strip_chr (label_with_underscore, '_');
 	nautilus_bonobo_set_label (ui,
-				   menu_item_path,
+				   command_path,
 				   label_with_underscore);
 	nautilus_bonobo_set_label (ui,
-				   command_path,
+				   toolitem_path,
 				   label_no_underscore);
 	
 	g_free (label_no_underscore);
@@ -619,12 +619,14 @@ Bonobo_RegistrationResult
 nautilus_bonobo_activation_register_for_display (const char    *iid,
 						 Bonobo_Unknown ref)
 {
-	char *real_iid;
+	const char *display_name;
+	GSList *reg_env ;
 	Bonobo_RegistrationResult result;
-
-	real_iid = eel_bonobo_make_registration_id (iid);
-	result = bonobo_activation_active_server_register (real_iid, ref);
-	g_free (real_iid);
-
+	
+	display_name = gdk_display_get_name (gdk_display_get_default());
+	reg_env = bonobo_activation_registration_env_set (NULL,
+							  "DISPLAY", display_name);
+	result = bonobo_activation_register_active_server (iid, ref, reg_env);
+	bonobo_activation_registration_env_free (reg_env);
 	return result;
 }
