@@ -617,10 +617,28 @@ nautilus_directory_remove_file (NautilusDirectory *directory, NautilusFile *file
 	}
 }
 
+#define NAUTILUS_DIRECTORY_FILE_LIST_DEFAULT_LIMIT 4000
+
 gboolean
 nautilus_directory_file_list_length_reached (NautilusDirectory *directory)
 {
-	return directory->details->confirmed_file_count >= NAUTILUS_DIRECTORY_FILE_LIST_HARD_LIMIT;
+	static gboolean inited = FALSE;
+	static int directory_limit = 0;
+
+	if (!inited) {
+		eel_preferences_add_auto_integer
+			("/apps/nautilus/preferences/directory_limit",
+			 &directory_limit);
+		inited = TRUE;
+	}
+	if (directory_limit < 0) {  /* unlimited */
+		return FALSE;
+	}
+	if (directory_limit == 0) { /* dead gconfd */
+		directory_limit = NAUTILUS_DIRECTORY_FILE_LIST_DEFAULT_LIMIT;
+	}
+
+	return directory->details->confirmed_file_count >= directory_limit;
 }
 
 GList *
@@ -821,26 +839,40 @@ call_files_added_free_list (gpointer key, gpointer value, gpointer user_data)
 	g_list_free (value);
 }
 
+static void
+call_files_changed_common (NautilusDirectory *directory, GList *file_list)
+{
+	GList *node;
+	NautilusFile *file;
+
+	for (node = file_list; node != NULL; node = node->next) {
+		file = node->data;
+		if (file->details->directory == directory) {
+			nautilus_directory_add_file_to_work_queue (directory, 
+								   file);
+		}
+	}
+	nautilus_directory_async_state_changed (directory);
+	nautilus_directory_emit_change_signals (directory, file_list);
+}
+
 static void
 call_files_changed_free_list (gpointer key, gpointer value, gpointer user_data)
 {
-	g_assert (NAUTILUS_IS_DIRECTORY (key));
 	g_assert (value != NULL);
 	g_assert (user_data == NULL);
 
-	nautilus_directory_emit_change_signals (key, value);
+	call_files_changed_common (NAUTILUS_DIRECTORY (key), value);
 	g_list_free (value);
 }
 
 static void
 call_files_changed_unref_free_list (gpointer key, gpointer value, gpointer user_data)
 {
-	g_assert (NAUTILUS_IS_DIRECTORY (key));
 	g_assert (value != NULL);
 	g_assert (user_data == NULL);
 
-	nautilus_directory_async_state_changed (key);
-	nautilus_directory_emit_change_signals (key, value);
+	call_files_changed_common (NAUTILUS_DIRECTORY (key), value);
 	nautilus_file_list_free (value);
 }
 
@@ -886,7 +918,7 @@ nautilus_directory_notify_files_added (GList *uris)
 	NautilusDirectory *directory;
 	GHashTable *parent_directories;
 	const char *uri;
-	const char *directory_uri;
+	char *directory_uri;
 	GnomeVFSURI *vfs_uri;
 	NautilusFile *file;
 
@@ -909,6 +941,7 @@ nautilus_directory_notify_files_added (GList *uris)
 			
 			directory_uri = uri_get_directory_part (uri);
 			file = nautilus_file_get_existing (directory_uri);
+			g_free (directory_uri);
 
 			if (file != NULL) {
 				nautilus_file_invalidate_count_and_mime_list (file);
@@ -926,14 +959,26 @@ nautilus_directory_notify_files_added (GList *uris)
 			continue;
 		}
 
-		/* Collect the URIs to use. */
-		vfs_uri = gnome_vfs_uri_new (uri);
-		if (vfs_uri == NULL) {
-			nautilus_directory_unref (directory);
-			g_warning ("bad uri %s", uri);
-			continue;
+		file = nautilus_file_get_existing (uri);
+		if (file) {
+			/* A file already exists, it was probably renamed.
+			 * If it was renamed this could be ignored, but 
+			 * queue a change just in case */
+			nautilus_file_changed (file);
+			nautilus_file_unref (file);
+		} else {
+			/* Collect the URIs to use. */
+			vfs_uri = gnome_vfs_uri_new (uri);
+			if (vfs_uri == NULL) {
+				nautilus_directory_unref (directory);
+				g_warning ("bad uri %s", uri);
+				continue;
+			}
+			
+			hash_table_list_prepend (added_lists, 
+						 directory, 
+						 vfs_uri);
 		}
-		hash_table_list_prepend (added_lists, directory, vfs_uri);
 		nautilus_directory_unref (directory);
 	}
 
@@ -967,10 +1012,13 @@ nautilus_directory_notify_files_changed (GList *uris)
 		/* Find the file. */
 		file = nautilus_file_get_existing (uri);
 		if (file != NULL) {
-			/* Tell it to re-get info and emit a changed
-			 * signal.
+			/* Tell it to re-get info now, and later emit
+			 * a changed signal.
 			 */
 			file->details->file_info_is_up_to_date = FALSE;
+			file->details->top_left_text_is_up_to_date = FALSE;
+			file->details->link_info_is_up_to_date = FALSE;
+
 			hash_table_list_prepend (changed_lists,
 						 file->details->directory,
 						 file);
@@ -1200,7 +1248,8 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 	GList *new_files_list, *unref_list;
 	GHashTable *added_lists, *changed_lists;
 	char *name;
-
+	GList *cancel_attributes;
+	
 	/* Make a list of added and changed files in each directory. */
 	new_files_list = NULL;
 	added_lists = g_hash_table_new (NULL, NULL);
@@ -1210,6 +1259,8 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 	/* Make a list of parent directories that will need their counts updated. */
 	parent_directories = g_hash_table_new (NULL, NULL);
 
+	cancel_attributes = nautilus_file_get_all_attributes ();
+
 	for (p = uri_pairs; p != NULL; p = p->next) {
 		pair = p->data;
 
@@ -1248,6 +1299,10 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 			old_directory = file->details->directory;
 			collect_parent_directories (parent_directories, old_directory);
 
+			/* Cancel loading of attributes in the old directory */
+			nautilus_directory_cancel_loading_file_attributes
+				(old_directory, file, cancel_attributes);
+
 			/* Locate the new directory. */
 			new_directory = get_parent_directory (pair->to_uri);
 			collect_parent_directories (parent_directories, new_directory);
@@ -1258,13 +1313,11 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 			 */
 			nautilus_directory_unref (new_directory);
 
-			/* Update the file's name. */
+			/* Update the file's name and directory. */
 			name = eel_uri_get_basename (pair->to_uri);
-			nautilus_file_update_name (file, name);
+			nautilus_file_update_name_and_directory 
+				(file, name, new_directory);
 			g_free (name);
-
-			/* Update the file's directory. */
-			nautilus_file_set_directory (file, new_directory);
 			
 			hash_table_list_prepend (changed_lists,
 						 old_directory,
@@ -1296,6 +1349,8 @@ nautilus_directory_notify_files_moved (GList *uri_pairs)
 	/* Separate handling for brand new file objects. */
 	nautilus_directory_notify_files_added (new_files_list);
 	g_list_free (new_files_list);
+
+	g_list_free (cancel_attributes);
 }
 
 void 
@@ -1390,6 +1445,7 @@ nautilus_directory_schedule_position_set (GList *position_setting_list)
 	const NautilusFileChangesQueuePosition *item;
 	NautilusFile *file;
 	char *position_string;
+	char *screen_string;
 
 	for (p = position_setting_list; p != NULL; p = p->next) {
 		item = (NautilusFileChangesQueuePosition *) p->data;
@@ -1408,6 +1464,14 @@ nautilus_directory_schedule_position_set (GList *position_setting_list)
 			 NULL,
 			 position_string);
 		g_free (position_string);
+
+		screen_string = g_strdup_printf ("%d", item->screen);
+		nautilus_file_set_metadata
+			(file,
+			 NAUTILUS_METADATA_KEY_SCREEN,
+			 NULL,
+			 screen_string);
+		g_free (screen_string);
 		
 		nautilus_file_unref (file);
 	}
