@@ -45,7 +45,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <libnautilus-private/egg-screen-exec.h>
+#include <libegg/egg-screen-exec.h>
 #include <libnautilus-private/nautilus-bonobo-extensions.h>
 #include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-directory-notify.h>
@@ -463,7 +463,7 @@ create_mount_link (FMDesktopIconView *icon_view,
 		break;
 	
 	case NAUTILUS_DEVICE_ZIP_DRIVE:
-		icon_name = "gome-fs-zipdisk";
+		icon_name = "gnome-dev-zipdisk";
 		break;
 
 	case NAUTILUS_DEVICE_APPLE:
@@ -665,8 +665,8 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 		desktop_directory_changed_callback (NULL);
 	}
 
+	fm_icon_view_filter_by_screen (FM_ICON_VIEW (desktop_icon_view), TRUE);
 	icon_container = get_icon_container (desktop_icon_view);
-
 	nautilus_icon_container_set_use_drop_shadows (icon_container, TRUE);
 	fm_icon_container_set_sort_desktop (FM_ICON_CONTAINER (icon_container), TRUE);
 
@@ -683,6 +683,7 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 	}
 	
 	nautilus_icon_container_set_is_fixed_size (icon_container, TRUE);
+	nautilus_icon_container_set_is_desktop (icon_container, TRUE);
 
 	/* Set up default mount black list */
 	list = g_list_prepend (NULL, g_strdup ("/proc"));
